from pathlib import Path

from framex._dicts._constants import _EXTENSION, _LOCAL_DIR

# find locally cached _EXTENSION files
_LOCAL_CACHES = {path.stem: path for path in Path(_LOCAL_DIR).glob(f"*.{_EXTENSION}")}



from pathlib import Path

_LOCAL_DIR = Path(__file__).parent.parent / "data"
if not _LOCAL_DIR.exists():
    _LOCAL_DIR.mkdir()

_EXTENSION = "parquet"
_REMOTE_DIR_GITHUB = f"https://github.com/datavil/datasets/raw/main/{_EXTENSION}/"
_INFO_FILE = "https://github.com/datavil/datasets/raw/main/datasets_info.csv"



from framex._dicts._github import _GITHUB_DATASETS
from framex._dicts._local import _LOCAL_CACHES

# more will be added later

# merge dictionaries
_REMOTE_DATASETS = {**_GITHUB_DATASETS}
_DATASETS = {**_LOCAL_CACHES, **_REMOTE_DATASETS}



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Callable, Literal

from polars import read_parquet

from framex._dicts import _REMOTE_DATASETS
from framex._dicts._constants import _EXTENSION

# from framex._dicts._constants import _EXTENSION, _LOCAL_DIR

if TYPE_CHECKING:
    from polars import DataFrame


def _save(
    frame: DataFrame,
    path: Path,
    format: str | Literal["feather", "parquet", "csv", "json", "ipc"] = _EXTENSION,
) -> None:
    """
    Saves a LazyFrame to the desired format.

    Parameters
    ----------
    frame : polars.LazyFrame
        The frame to save.
    name : str
        The name of the dataset.
    path : Path[str]
        The path to save the dataset to.
    format : str or Literal["feather", "parquet", "csv", "json", "ipc"], optional
        The format to save the dataset in.
        Default is "feather"
    """
    if format in ("feather", "ipc"):
        frame.write_ipc(path, compression="zstd")
    elif format == "parquet":
        frame.write_parquet(path, compression="zstd")
    elif format == "csv":
        frame.write_csv(path)
    elif format == "json":
        frame.write_ndjson(path)


def get(
    name: str,
    *,
    dir: str | Path | None = None,
    overwrite: bool = False,
    format: str | Literal["feather", "parquet", "csv", "json", "ipc"] = _EXTENSION,
) -> None:
    """
    Loads dataset by with the given name if available.

    Parameters
    ----------
    name : str
        Name of the dataset to load.

    dir : str, optional
        Directory to save the dataset to.

    overwrite : bool, optional
        Whether to overwrite the dataset if it already exists.
        Default is True

    format : str or Literal["feather", "parquet", "csv", "json", "ipc"], optional
        The format to save the dataset in.
        Default is "feather"

    Returns
    -------
    None
    """
    if dir is None:
        dir = Path().resolve()
    else:
        dir = Path(dir).resolve()
    # select the function to load the dataset
    loader: Callable[..., DataFrame] = read_parquet

    # check if the dataset is available
    if name not in _REMOTE_DATASETS:
        msg = f"Dataset {name} not found."
        raise ValueError(msg)
    else:
        frame = loader(_REMOTE_DATASETS.get(name))

    # list of all files in the directory
    dir = Path(dir).resolve()
    cached = list(dir.glob(f"*{format}"))
    path = dir / f"{name}.{format}"

    if path in cached:
        if not overwrite:
            msg = f"File {path} already exists.\nUse `--overwrite` to overwrite."
            print(msg)
            return
        else:
            print(f"Overwriting: {path}")
            _save(frame=frame, path=path, format=format)
    elif path not in cached:
        _save(frame=frame, path=path, format=format)
        print(f"Saving: {path}")

    return


if __name__ == "__main__":
    pass



import argparse

from framex.datasets import available
from framex.datasets._cli import get


def main():
    parser = argparse.ArgumentParser(description="Framex CLI")
    subparsers = parser.add_subparsers(dest="command")

    # Subcommand: get
    get_parser = subparsers.add_parser("get", help="Get dataset(s)")
    get_parser.add_argument(
        "datasets", nargs="+", help="Dataset name(s), accepts multiple names"
    )
    get_parser.add_argument(
        "--dir",
        "-d",
        help="Directory to save the dataset to. Defaults to current directory.",
        default=".",
        metavar="",
    )
    get_parser.add_argument(
        "--format",
        "-f",
        help="Format to save the dataset in. Defaults to csv.",
        default="csv",
        metavar="",
    )
    get_parser.add_argument(
        "--overwrite",
        "-o",
        action="store_true",
        help="Whether to overwrite the dataset if it already exists.",
    )

    # Subcommand: list
    list_parser = subparsers.add_parser("list", help="List available datasets")
    args = parser.parse_args()
    if args.command is None:
        parser.print_help()  # Show help if no command is provided
        return

    elif args.command == "get":
        for dataset in args.datasets:
            get(
                name=dataset,
                dir=args.dir,
                format=args.format,
                overwrite=args.overwrite,
            )
    elif args.command == "list":
        print(available(option="remote")["remote"])
    else:
        msg = f"Invalid command: {args.command}"
        raise ValueError(msg)


if __name__ == "__main__":
    main()



from __future__ import annotations

from typing import TYPE_CHECKING, Callable, Literal

import polars
from polars import read_ipc, read_parquet, scan_ipc, scan_parquet

from framex._dicts import _DATASETS, _LOCAL_CACHES, _REMOTE_DATASETS
from framex._dicts._constants import _EXTENSION, _INFO_FILE, _LOCAL_DIR

if TYPE_CHECKING:
    from polars import DataFrame, LazyFrame


def load(
    name: str,
    *,
    lazy: bool = False,
    check_local: bool = True,
    cache: bool = True,
) -> DataFrame | LazyFrame:
    """
    Loads dataset by with the given name if available.

    Parameters
    ----------
    name : str
        Name of the dataset to load.
    lazy : bool, optional
        Lazy loading,
        Default is  False

    check_local : bool, optional
        Whether to check if the dataset is available locally
        Default is True

    cache : bool, optional
        Whether to cache the dataset locally.
        Default is True

    Returns
    -------
    polars.DataFrame or polars.LazyFrame
    """
    # select the function to load the dataset
    if _EXTENSION == "parquet":
        loader: Callable[..., DataFrame | LazyFrame] = scan_parquet if lazy else read_parquet
    # elif _EXTENSION == "feather":
    #     loader: Callable[..., DataFrame | LazyFrame] = scan_ipc if lazy else read_ipc

    # check if the dataset is available
    if name not in _DATASETS:
        msg = f"Dataset {name} not found."
        raise ValueError(msg)

    # local_path = _LOCAL_CACHES.get(name)
    # load the dataset
    if check_local:
        if name in _LOCAL_CACHES:
            frame = loader(_LOCAL_CACHES.get(name))
        else:
            frame = loader(_DATASETS.get(name))
    else:
        frame = loader(_DATASETS.get(name))

    # cache the dataset locally
    if cache and name not in _LOCAL_CACHES:
        if lazy:
            # sink IPC is not yet supportted
            # polars.exceptions.InvalidOperationError:
            # sink_Ipc(IpcWriterOptions { compression: Some(ZSTD), maintain_order: true })
            # not yet supported in standard engine.
            # Use 'collect().write_parquet()'
            frame.sink_parquet(_LOCAL_DIR / f"{name}.{_EXTENSION}", compression="zstd")
        else:
            frame.write_parquet(_LOCAL_DIR / f"{name}.{_EXTENSION}", compression="zstd")

    return frame


def available(option: str | None = None) -> dict[str, list[str]]:
    """
    List available datasets.

    Parameters
    ----------
    option : str, optional {"remote", "local"}
        The option to list available datasets.
        Default is None

    Returns
    -------
    list of available datasets

    Raises
    ------
    ValueError
        If the option is not 'remote' or 'local'.

    """
    if option is None:
        return {
            "remote": list(_REMOTE_DATASETS.keys()),
            "local": list(_LOCAL_CACHES.keys()),
        }
    elif option == "remote":
        return {option: list(_REMOTE_DATASETS.keys())}
    elif option == "local":
        return {option: list(_LOCAL_CACHES.keys())}
    else:
        msg = "Invalid option. Please use either 'remote' or 'local'."
        raise ValueError(msg)

def about(name: str, mode: Literal["print", "row"] = "print") -> None | polars.DataFrame:
    """
    Print or return information about a dataset.

    Parameters
    ----------
    name : str
        Name of the dataset.
    mode : Literal["print", "row"]
        The mode to print information.
        Default is "print"
            print: prints the information
            row: returns the information as a single row polars.DataFrame

    Returns
    -------
    None
    """
    df = polars.read_csv(_INFO_FILE)
    try:
        row = df.filter(polars.col("name")==name)
    except Exception as e:
        msg = f"Dataset {name} not found in datasets_info.csv"
        raise ValueError(msg) from e

    if mode == "row":#
        return row
    elif mode == "print":
        og_name = row.select("source").item().split('/')[-1]
        og_id = "OG NAME"
        for column in row.columns:
            print(f"{column.upper():<8}: {row.select(column).item()}")
        print(f"{og_id:<8}: {og_name}")
        return
    else:
        msg = "Invalid mode. Please use either 'print' or 'row'."
        raise ValueError(msg)




from framex._dicts._all import _DATASETS, _REMOTE_DATASETS
from framex._dicts._constants import (
    _EXTENSION,
    _INFO_FILE,
    _LOCAL_DIR,
    _REMOTE_DIR_GITHUB,
)
from framex._dicts._github import _GITHUB_DATASETS
from framex._dicts._local import _LOCAL_CACHES

__all__ = [
    "_EXTENSION",
    "_LOCAL_DIR",
    "_REMOTE_DIR_GITHUB",
    "_GITHUB_DATASETS",
    "_LOCAL_CACHES",
    "_DATASETS",
    "_REMOTE_DATASETS",
    "_INFO_FILE",
]



from __future__ import annotations

import requests

from framex._dicts._constants import _EXTENSION

# GitHub API URL for the directory contents
_API_URL = f"https://api.github.com/repos/datavil/datasets/contents/{_EXTENSION}"


def _get_names(api_url: str) -> dict[str, str]:
    """
    Get the names of the datasets from the GitHub API.

    Parameters
    ----------
    api_url : str
        The URL of the GitHub API.

    Returns
    -------
    list of str
        The names of the datasets.
    """
    response = requests.get(api_url)
    files = response.json()

    # Extract .feather files
    datasets = {
        file["name"].rstrip(f".{_EXTENSION}"): file["download_url"]
        for file in files
        if file["name"].endswith(f".{_EXTENSION}")
    }

    return datasets


_GITHUB_DATASETS = _get_names(_API_URL)
